#-------------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-------------------------------------------------------------------------------

require 'json'

Sketchup.require 'tt_vertex2/app_settings'
Sketchup.require 'tt_vertex2/logger'


module TT::Plugins::VertexTools2
  module Update

    def self.check(force: false)
      # Consider switching to HTTPS.
      # http://stackoverflow.com/a/12883718/486990

      # Only check once per session.
      @checked = true
	  return false

      server = SETTINGS.server
      uri = "#{server}/api/v1/product/#{EXTENSION[:product_id]}/version"

      data = {
          :version => EXTENSION[:version],
          :license => License.read[:data],
          :sketchup => {
              :version => Sketchup.version,
              :bitness => POINTER_SIZE,
              :platform => PLATFORM_ID,
              :pro => Sketchup.is_pro?
          }
      }

      # noinspection RubyStringKeysInHashInspection
      headers = {
          'Content-Type' => 'application/json',
          # The following fields are used to indicate to Laravel that this is
          # an AJAX request and should get HTTP 422 and JSON response upon
          # validation errors.
          'Accept' => 'application/json',
          'X-Requested-With' => "SketchUp #{Sketchup.version}"
      }

      @request = Sketchup::Http::Request.new(uri, Sketchup::Http::POST)
      @request.headers = headers
      @request.body = JSON.pretty_generate(data)
      @request.start do |request, response|
        Log.debug "#{self} (Response)"
        status_name = self.code_to_name(response.status_code)
        Log.debug "#{status_name} (#{response.status_code})"
        if response.status_code == 204 # 204 HTTPNoContent
          Log.debug '> No updates available...'
        elsif (200..299).include?(response.status_code) # 2xx HTTPSuccess
          Log.debug response.body
          data = JSON.parse(response.body)
          self.handle(data)
        elsif response.status_code == 0
          Log.error "Check for #{EXTENSION[:name]} update timed out."
        else
          Log.error '> Failed to check for update...'
          Log.error response.body
        end
      end
      true
    end

    def self.available?
      version = SETTINGS.read(:update_version)
      return false if version.nil?
      Gem::Version.new(version) > Gem::Version.new(EXTENSION[:version])
    end

    def self.version
      SETTINGS.read(:update_version) or raise
    end

    def self.timestamp
      SETTINGS.read(:update_timestamp) or raise
    end

    # @private
    # @param [Hash] data
    def self.handle(data)
      SETTINGS.write(:update_version, data['version'])
      SETTINGS.write(:update_timestamp, data['timestamp'])
      nil
    end

    # @private
    # @param [Integer] code
    # @return [String]
    def self.code_to_name(code)
      return 'Timeout' if code == 0
      CODE_TO_NAME[code.to_s] || 'Unknown'
    rescue => error
      warn error.message
      return 'ERROR'
    end

    # Copied from https://www.rubydoc.info/stdlib/net/Net/HTTPResponse
    CODE_TO_NAME = {
      '100' => 'HTTPContinue',
      '101' => 'HTTPSwitchProtocol',

      '200' => 'HTTPOK',
      '201' => 'HTTPCreated',
      '202' => 'HTTPAccepted',
      '203' => 'HTTPNonAuthoritativeInformation',
      '204' => 'HTTPNoContent',
      '205' => 'HTTPResetContent',
      '206' => 'HTTPPartialContent',
      '207' => 'HTTPMultiStatus',
      '226' => 'HTTPIMUsed',

      '300' => 'HTTPMultipleChoices',
      '301' => 'HTTPMovedPermanently',
      '302' => 'HTTPFound',
      '303' => 'HTTPSeeOther',
      '304' => 'HTTPNotModified',
      '305' => 'HTTPUseProxy',
      '307' => 'HTTPTemporaryRedirect',
      '308' => 'HTTPPermanentRedirect',

      '400' => 'HTTPBadRequest',
      '401' => 'HTTPUnauthorized',
      '402' => 'HTTPPaymentRequired',
      '403' => 'HTTPForbidden',
      '404' => 'HTTPNotFound',
      '405' => 'HTTPMethodNotAllowed',
      '406' => 'HTTPNotAcceptable',
      '407' => 'HTTPProxyAuthenticationRequired',
      '408' => 'HTTPRequestTimeOut',
      '409' => 'HTTPConflict',
      '410' => 'HTTPGone',
      '411' => 'HTTPLengthRequired',
      '412' => 'HTTPPreconditionFailed',
      '413' => 'HTTPRequestEntityTooLarge',
      '414' => 'HTTPRequestURITooLong',
      '415' => 'HTTPUnsupportedMediaType',
      '416' => 'HTTPRequestedRangeNotSatisfiable',
      '417' => 'HTTPExpectationFailed',
      '422' => 'HTTPUnprocessableEntity',
      '423' => 'HTTPLocked',
      '424' => 'HTTPFailedDependency',
      '426' => 'HTTPUpgradeRequired',
      '428' => 'HTTPPreconditionRequired',
      '429' => 'HTTPTooManyRequests',
      '431' => 'HTTPRequestHeaderFieldsTooLarge',

      '500' => 'HTTPInternalServerError',
      '501' => 'HTTPNotImplemented',
      '502' => 'HTTPBadGateway',
      '503' => 'HTTPServiceUnavailable',
      '504' => 'HTTPGatewayTimeOut',
      '505' => 'HTTPVersionNotSupported',
      '507' => 'HTTPInsufficientStorage',
      '511' => 'HTTPNetworkAuthenticationRequired',
    }.freeze

  end # module Update
end # module
